SystemOrganization addCategory: #'Athens-Balloon'!

AthensSurface subclass: #AthensBalloonSurface
	instanceVariableNames: 'form mask maskEnabled pathTransform paintTransform imageTransform fillTransform strokeTransform engine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Balloon'!

!AthensBalloonSurface commentStamp: '<historical>' prior: 0!
I am concrete implementation of Athens surface which using balloon engine for rendering. !

!AthensBalloonSurface class methodsFor: 'as yet unclassified' stamp: 'sig 2/29/2012 21:45'!
extent: aPoint
	"Create a new surface with given extent."
	
	^ self new form: (Form extent: aPoint depth: 32)! !

!AthensBalloonSurface methodsFor: 'converting' stamp: 'NicolaiHess 6/24/2014 22:56'!
asAthensPaintOn: aCanvas
	^ AthensBalloonImagePaint new form: self asForm; yourself! !

!AthensBalloonSurface methodsFor: 'converting' stamp: 'sig 2/29/2012 21:52'!
asForm
	^ form! !

!AthensBalloonSurface methodsFor: 'drawing' stamp: 'sig 2/29/2012 21:49'!
clear
	^ self clearForm! !

!AthensBalloonSurface methodsFor: 'testing' stamp: 'IgorStasenko 4/1/2011 09:36'!
clearForm

	form bits: ( Bitmap new: form bits size).
	! !

!AthensBalloonSurface methodsFor: 'clipping' stamp: 'NicolaiHess 6/24/2014 22:57'!
clipBy: aRectangle during: aBlock

	| oldRect transformedClip|
	transformedClip := (pathTransform transform:aRectangle origin) corner: (pathTransform transform:aRectangle corner).
	oldRect := engine clipRect.
	engine clipRect: (oldRect intersect: transformedClip).

	aBlock ensure: [ engine clipRect: oldRect ]! !

!AthensBalloonSurface methodsFor: 'accessing' stamp: 'IgorStasenko 11/21/2011 16:33'!
clipRect
	^ engine clipRect! !

!AthensBalloonSurface methodsFor: 'paints' stamp: 'IgorStasenko 4/14/2011 17:12'!
createCircularGradient: colorRamp center: aCenter radius: aRadius

	^ AthensBalloonRadialGradient new
		colorRamp: colorRamp;
		center: aCenter;
		radius: aRadius;
		yourself! !

!AthensBalloonSurface methodsFor: 'paints' stamp: 'IgorStasenko 3/8/2012 13:48'!
createFormPaint: aForm  
	^ AthensBalloonImagePaint new
		form: aForm! !

!AthensBalloonSurface methodsFor: 'paints' stamp: 'IgorStasenko 11/30/2013 08:22'!
createLinearGradient: colorRamp origin: pt1 corner: pt2
 	self 
		deprecated: 'Use #createLinearGradient:start:stop: instead'
		on: 	'12 April 2013'
		in: 'ConfigurationOfAthens 2.0'.
		
	^ self createLinearGradient: colorRamp origin: pt1 corner: pt2
! !

!AthensBalloonSurface methodsFor: 'paints' stamp: 'NicolaiHess 6/24/2014 20:59'!
createLinearGradient: colorRamp start: pt1 stop: pt2
 
	^ AthensBalloonLinearGradient new
		colorRamp: colorRamp;
		origin: pt1;
		corner: pt2;
		yourself! !

!AthensBalloonSurface methodsFor: 'paths' stamp: 'IgorStasenko 4/18/2013 04:46'!
createPath: aPathCreatingBlock
	"Create a path from provided path builder instance"
	
	| builder |
	builder := AthensSimplePathBuilder new.
	aPathCreatingBlock value: builder.

	^ AthensBalloonPath new convertFromBuilder: builder.
	
	! !

!AthensBalloonSurface methodsFor: 'paints' stamp: 'NicolaiHess 6/24/2014 21:00'!
createRadialGradient: colorRamp center: aCenter radius: aRadius focalPoint: fp
	^ self createCircularGradient: colorRamp center: aCenter radius: aRadius! !

!AthensBalloonSurface methodsFor: 'paints' stamp: 'IgorStasenko 3/22/2011 21:59'!
createSolidColorPaint: aColor 
	^ AthensBalloonSolidColorPaint new color: aColor! !

!AthensBalloonSurface methodsFor: 'paints' stamp: 'IgorStasenko 4/27/2012 11:22'!
createStrokePaintFor: aPaint

	^ AthensBalloonStrokePaint new fillPaint: aPaint! !

!AthensBalloonSurface methodsFor: 'masking' stamp: 'IgorStasenko 3/27/2011 18:16'!
disableMask
	maskEnabled := false.! !

!AthensBalloonSurface methodsFor: 'drawing' stamp: 'IgorStasenko 4/21/2011 16:35'!
drawImage: aForm 

	engine drawImage: aForm transform: imageTransform! !

!AthensBalloonSurface methodsFor: 'masking' stamp: 'IgorStasenko 3/27/2011 18:16'!
enableMask
	maskEnabled := true! !

!AthensBalloonSurface methodsFor: 'accessing' stamp: 'IgorStasenko 3/27/2011 20:30'!
engine
	^ engine! !

!AthensBalloonSurface methodsFor: 'drawing' stamp: 'IgorStasenko 4/1/2011 09:25'!
fillBezierShape: contours color: aColor 

	engine
		prepareForRendering;
		registerColorFill: aColor	;
		fillBezierShape: contours! !

!AthensBalloonSurface methodsFor: 'drawing' stamp: 'IgorStasenko 9/14/2011 15:08'!
fillBezierShape: anArray color: aColor width: aSmallInteger 
	self shouldBeImplemented.! !

!AthensBalloonSurface methodsFor: 'rendering dispatch' stamp: 'NicolaiHess 6/24/2014 21:10'!
fillPath: aPath withSolidColor: aColor
	self fillBezierShape: aPath contoursForFilling color: aColor! !

!AthensBalloonSurface methodsFor: 'drawing' stamp: 'IgorStasenko 4/1/2011 09:24'!
fillRectangle: rect color: aColor

	engine
		prepareForRendering;
		registerColorFill: aColor	;
		fillRectangle: rect
! !

!AthensBalloonSurface methodsFor: 'rendering dispatch' stamp: 'NicolaiHess 6/24/2014 21:04'!
fillRectangle: aRectangle withSolidColor: aColor
	self fillRectangle: aRectangle color: aColor! !

!AthensBalloonSurface methodsFor: 'accessing matrices' stamp: 'IgorStasenko 3/26/2011 02:12'!
fillTransform

	^ fillTransform! !

!AthensBalloonSurface methodsFor: 'accessing matrices' stamp: 'IgorStasenko 3/26/2011 02:12'!
fillTransform: anObject

	fillTransform := anObject! !

!AthensBalloonSurface methodsFor: 'accessing' stamp: 'IgorStasenko 3/21/2011 15:10'!
form
	^ form! !

!AthensBalloonSurface methodsFor: 'accessing' stamp: 'IgorStasenko 4/21/2011 16:17'!
form: aForm

	form := aForm.
	mask := Form extent: form extent depth: 1. 

	engine := AthensBalloonEngine new.
	engine bitBlt: (GrafPort toForm: form) .
	engine clipRect: (0@0 extent: form extent).
	engine edgeTransform: pathTransform.
	engine destOffset: 0@0.
	engine reset.! !

!AthensBalloonSurface methodsFor: 'fonts' stamp: 'IgorStasenko 10/20/2011 10:03'!
getFreetypeFontRendererFor: aFreeTypeFont 
	
	^ (FT2GlyphRenderer forFont: aFreeTypeFont surface: self)! !

!AthensBalloonSurface methodsFor: 'accessing matrices' stamp: 'IgorStasenko 3/26/2011 02:12'!
imageTransform

	^ imageTransform! !

!AthensBalloonSurface methodsFor: 'initialize-release' stamp: 'IgorStasenko 3/21/2012 19:12'!
initialize
	pathTransform := AthensAffineTransform new.
	paintTransform := AthensAffineTransform new.
	
	imageTransform := AthensAffineTransform new " AthensGenericTransform new".
	fillTransform := AthensAffineTransform new.
	strokeTransform := AthensAffineTransform new.
	maskEnabled := false.! !

!AthensBalloonSurface methodsFor: 'masking' stamp: 'IgorStasenko 3/27/2011 18:17'!
maskEnabled
	^ maskEnabled! !

!AthensBalloonSurface methodsFor: 'accessing' stamp: 'IgorStasenko 3/21/2012 19:12'!
paintTransform 
	^ paintTransform ! !

!AthensBalloonSurface methodsFor: 'accessing matrices' stamp: 'IgorStasenko 3/26/2011 02:12'!
pathTransform

	^ pathTransform! !

!AthensBalloonSurface methodsFor: 'accessing matrices' stamp: 'IgorStasenko 10/17/2011 13:52'!
pathTransform: anObject

	pathTransform := anObject copy.
	engine edgeTransform: pathTransform.! !

!AthensBalloonSurface methodsFor: 'drawing' stamp: 'IgorStasenko 9/14/2011 15:12'!
strokeBezierShape: contours color: aColor width: aWidth 
	
	engine
		prepareForRendering;
		registerColorFill: aColor	;
		strokeBezierShape: contours width: aWidth! !

!AthensBalloonSurface methodsFor: 'drawing' stamp: 'IgorStasenko 9/14/2011 15:24'!
strokeRectangle: rect color: aColor width: aWidth
	 
	engine
		prepareForRendering;
		registerColorFill: aColor	;
		strokeRectangle: rect width: aWidth! !

!AthensBalloonSurface methodsFor: 'accessing matrices' stamp: 'IgorStasenko 3/26/2011 02:12'!
strokeTransform

	^ strokeTransform! !

!AthensBalloonSurface methodsFor: 'accessing matrices' stamp: 'IgorStasenko 3/26/2011 02:12'!
strokeTransform: anObject

	strokeTransform := anObject! !

AthensPaint subclass: #AthensBalloonImagePaint
	instanceVariableNames: 'form origin direction tileFlag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Balloon'!

!AthensBalloonImagePaint commentStamp: '<historical>' prior: 0!
i representing an image (form) paint  !

!AthensBalloonImagePaint methodsFor: 'rendering' stamp: 'IgorStasenko 8/30/2013 16:32'!
athensFillPath: aPath on: anAthensCanvas

	anAthensCanvas surface engine
		prepareForRendering;
		registerFormFill: form origin: origin direction: direction normal: self normal;
		fillBezierShape: aPath contoursForFilling! !

!AthensBalloonImagePaint methodsFor: 'rendering' stamp: 'NicolaiHess 6/24/2014 23:48'!
athensFillRectangle: aRect on: anAthensCanvas
	"This is a terminal method in rendering dispatch scheme
	canvas->shape->paint. See AthensCanvas>>fillShape: "
	|rotation normal|
	rotation := anAthensCanvas surface paintTransform transform: direction.
	normal := anAthensCanvas surface paintTransform transform:self normal.
	

	anAthensCanvas surface engine reset
		prepareForRendering;
		registerFormFill: form origin: origin direction: rotation normal: normal tileFlag: tileFlag;
		fillRectangle: aRect
! !

!AthensBalloonImagePaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/21/2011 15:45'!
direction

	^ direction! !

!AthensBalloonImagePaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/21/2011 15:45'!
direction: anObject

	direction := anObject! !

!AthensBalloonImagePaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/21/2011 15:30'!
form

	^ form! !

!AthensBalloonImagePaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/21/2011 15:49'!
form: anObject

	form := anObject.
	origin := 0@0.
	direction := form extent x @0.! !

!AthensBalloonImagePaint methodsFor: 'initialize' stamp: 'NicolaiHess 6/24/2014 23:17'!
initialize

	super initialize.

	direction := 0@0.
	form := nil.
	origin := 0@0.
	tileFlag := false.! !

!AthensBalloonImagePaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/21/2011 15:47'!
normal
	^ ((direction y negated @ direction x) *  form height / form width ) rounded! !

!AthensBalloonImagePaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/21/2011 15:45'!
origin

	^ origin! !

!AthensBalloonImagePaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/21/2011 15:45'!
origin: anObject

	origin := anObject! !

!AthensBalloonImagePaint methodsFor: 'accessing' stamp: 'NicolaiHess 6/24/2014 23:17'!
repeat
	tileFlag:=true! !

AthensPaint subclass: #AthensBalloonPaint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Balloon'!

!AthensBalloonPaint commentStamp: '<historical>' prior: 0!
i am an abstract root of many balloon-specific paints!

AthensBalloonPaint subclass: #AthensBalloonGradientPaint
	instanceVariableNames: 'colorRamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Balloon'!

!AthensBalloonGradientPaint commentStamp: '<historical>' prior: 0!
my subclasses used internally in balloon backend for Athens to represenet gradient paints.!

!AthensBalloonGradientPaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/14/2011 16:27'!
colorRamp

	^ colorRamp! !

!AthensBalloonGradientPaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/14/2011 16:27'!
colorRamp: anObject

	colorRamp := anObject! !

!AthensBalloonGradientPaint methodsFor: 'rendering' stamp: 'IgorStasenko 9/19/2011 15:34'!
strokeRect: aRectangle on: anAthensCanvas stroke: anAthensBalloonStrokePaint 
	self shouldBeImplemented.! !

!AthensBalloonGradientPaint methodsFor: 'rendering' stamp: 'NicolaiHess 6/24/2014 22:46'!
strokeRect: aRectangle on: anAthensCanvas strokeWidth: strokeWidth
	self shouldBeImplemented.! !

AthensBalloonGradientPaint subclass: #AthensBalloonLinearGradient
	instanceVariableNames: 'origin corner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Balloon'!

!AthensBalloonLinearGradient commentStamp: '<historical>' prior: 0!
i represent a linear gradient paint in balloon backend.!

!AthensBalloonLinearGradient methodsFor: 'rendering' stamp: 'IgorStasenko 8/30/2013 16:32'!
athensFillPath: aPath on: anAthensCanvas
	| torigin tcorner |

	"A gradient fill coordinates are first transformed by 
	paintTransform matrix,
	then
	by pathTransform matrix"
		
	torigin := anAthensCanvas paintTransform transform: origin.
	tcorner :=  anAthensCanvas paintTransform transform: corner.

	^ anAthensCanvas surface engine
		prepareForRendering;
		registerLinearGradient: colorRamp origin: torigin corner: tcorner;
		fillBezierShape: aPath contoursForFilling! !

!AthensBalloonLinearGradient methodsFor: 'rendering' stamp: 'IgorStasenko 8/30/2013 16:35'!
athensFillRectangle: aRect on: anAthensCanvas
	"This is a terminal method in rendering dispatch scheme
	canvas->shape->paint. See AthensCanvas>>fillShape: "
	
	| torigin tcorner |

	"A gradient fill coordinates are first transformed by 
	paintTransform matrix,
	then
	by pathTransform matrix"
		
	torigin := anAthensCanvas paintTransform transform: origin.
	tcorner :=  anAthensCanvas paintTransform transform: corner.

	^ anAthensCanvas surface engine
		prepareForRendering;
		registerLinearGradient: colorRamp origin: torigin corner: tcorner;
		fillRectangle: aRect! !

!AthensBalloonLinearGradient methodsFor: 'accessing' stamp: 'IgorStasenko 4/1/2011 10:47'!
corner

	^ corner! !

!AthensBalloonLinearGradient methodsFor: 'accessing' stamp: 'IgorStasenko 4/1/2011 10:47'!
corner: anObject

	corner := anObject! !

!AthensBalloonLinearGradient methodsFor: 'accessing' stamp: 'IgorStasenko 4/1/2011 10:47'!
origin

	^ origin! !

!AthensBalloonLinearGradient methodsFor: 'accessing' stamp: 'IgorStasenko 4/1/2011 10:47'!
origin: anObject

	origin := anObject! !

!AthensBalloonLinearGradient methodsFor: 'rendering' stamp: 'IgorStasenko 3/8/2012 13:59'!
strokeRect: rect on: canvas stroke: strokePaint

	| torigin tcorner |

	"A gradient fill coordinates are first transformed by 
	paintTransform matrix,
	then
	by pathTransform matrix"
		
	torigin := canvas paintTransform transform: origin.
	tcorner :=  canvas paintTransform transform: corner.

	^ canvas surface engine
		prepareForRendering;
		registerLinearGradient: colorRamp origin: torigin corner: tcorner;
		strokeRectangle: rect width: strokePaint width! !

!AthensBalloonLinearGradient methodsFor: 'rendering' stamp: 'NicolaiHess 6/24/2014 22:47'!
strokeRect: rect on: canvas strokeWidth: strokeWidth

	| torigin tcorner |

	"A gradient fill coordinates are first transformed by 
	paintTransform matrix,
	then
	by pathTransform matrix"
		
	torigin := canvas paintTransform transform: origin.
	tcorner :=  canvas paintTransform transform: corner.

	^ canvas surface engine
		prepareForRendering;
		registerLinearGradient: colorRamp origin: torigin corner: tcorner;
		strokeRectangle: rect width: strokeWidth! !

AthensBalloonGradientPaint subclass: #AthensBalloonRadialGradient
	instanceVariableNames: 'center radius'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Balloon'!

!AthensBalloonRadialGradient commentStamp: '<historical>' prior: 0!
i represent a radial gradient paint in balloon backend.!

!AthensBalloonRadialGradient methodsFor: 'rendering' stamp: 'IgorStasenko 8/30/2013 16:32'!
athensFillPath: aPath on: anAthensCanvas
	"This is a terminal method in rendering dispatch scheme
	canvas->shape->paint. See AthensCanvas>>fillShape: "
	
	| c |

	c := anAthensCanvas paintTransform transform: center.

	^ anAthensCanvas surface engine
		prepareForRendering;
		registerRadialGradient: colorRamp center: c radius: radius;
		fillBezierShape: aPath contoursForFilling
! !

!AthensBalloonRadialGradient methodsFor: 'rendering' stamp: 'IgorStasenko 8/30/2013 16:35'!
athensFillRectangle: aRect on: anAthensCanvas
	"This is a terminal method in rendering dispatch scheme
	canvas->shape->paint. See AthensCanvas>>fillShape: "
	
	| c dir n |

	c := anAthensCanvas paintTransform transform: center.
	dir :=  (radius@0).
	n := (0@radius).
	
	^ anAthensCanvas surface engine
		prepareForRendering;
		registerRadialGradient: colorRamp center: c direction: dir normal: n;
		fillRectangle: aRect! !

!AthensBalloonRadialGradient methodsFor: 'accessing' stamp: 'IgorStasenko 4/14/2011 16:27'!
center

	^ center! !

!AthensBalloonRadialGradient methodsFor: 'accessing' stamp: 'IgorStasenko 4/14/2011 16:27'!
center: anObject

	center := anObject! !

!AthensBalloonRadialGradient methodsFor: 'accessing' stamp: 'IgorStasenko 4/14/2011 16:27'!
radius

	^ radius! !

!AthensBalloonRadialGradient methodsFor: 'accessing' stamp: 'IgorStasenko 4/14/2011 16:27'!
radius: anObject

	radius := anObject! !

!AthensBalloonPaint methodsFor: 'converting' stamp: 'IgorStasenko 3/21/2012 19:11'!
asAthensPaintOn: surf
	^ self ! !

!AthensBalloonPaint methodsFor: 'converting' stamp: 'IgorStasenko 4/27/2012 11:18'!
asStrokePaintOn: aCanvas
	^ AthensBalloonStrokePaint new fillPaint: self! !

AthensBalloonPaint subclass: #AthensBalloonSolidColorPaint
	instanceVariableNames: 'color'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Balloon'!

!AthensBalloonSolidColorPaint commentStamp: 'IgorStasenko 12/20/2011 15:27' prior: 0!
maybe Color could act as a AthensPaint by itself by implementing a corresponding protocol!

!AthensBalloonSolidColorPaint methodsFor: 'rendering' stamp: 'IgorStasenko 8/30/2013 16:32'!
athensFillPath: athensBalloonPath on: anAthensCanvas 

	^ anAthensCanvas surface 
		fillBezierShape: athensBalloonPath contoursForFilling  
		color: color.
		! !

!AthensBalloonSolidColorPaint methodsFor: 'rendering' stamp: 'IgorStasenko 8/30/2013 16:35'!
athensFillRectangle: aRect on: canvas
	"This is a terminal method in rendering dispatch scheme
	canvas->shape->paint. See AthensCanvas>>fillShape: "
	

	^ canvas surface fillRectangle: aRect color: color! !

!AthensBalloonSolidColorPaint methodsFor: 'accessing' stamp: 'IgorStasenko 3/22/2011 21:59'!
color

	^ color! !

!AthensBalloonSolidColorPaint methodsFor: 'accessing' stamp: 'IgorStasenko 3/22/2011 21:59'!
color: anObject

	color := anObject! !

!AthensBalloonSolidColorPaint methodsFor: 'rendering' stamp: 'IgorStasenko 9/14/2011 15:10'!
strokePath: aPath on: anAthensCanvas stroke: strokePaint 
	
	 anAthensCanvas surface 
		strokeBezierShape: aPath contoursForStroke
		color: color
		width: strokePaint width.
		! !

!AthensBalloonSolidColorPaint methodsFor: 'rendering' stamp: 'NicolaiHess 6/24/2014 22:51'!
strokePath: aPath on: anAthensCanvas strokeWidth: strokeWidth
	
	 anAthensCanvas surface 
		strokeBezierShape: aPath contoursForStroke
		color: color
		width: strokeWidth.
		! !

!AthensBalloonSolidColorPaint methodsFor: 'rendering' stamp: 'IgorStasenko 9/14/2011 15:17'!
strokeRect: rect on: canvas stroke: strokePaint

	^ canvas surface strokeRectangle: rect color: color width: strokePaint width! !

!AthensBalloonSolidColorPaint methodsFor: 'as yet unclassified' stamp: 'NicolaiHess 6/24/2014 22:53'!
strokeRect: rect on: canvas strokeWidth: strokeWidth

	^ canvas surface strokeRectangle: rect color: color width: strokeWidth! !

AthensStrokePaint subclass: #AthensBalloonStrokePaint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Balloon'!

!AthensBalloonStrokePaint commentStamp: '<historical>' prior: 0!
i represent stroke paint for balloon backend!

!AthensBalloonStrokePaint methodsFor: 'rendering' stamp: 'NicolaiHess 6/24/2014 22:48'!
athensFillPath: aPath on: aCanvas

	^ fillPaint strokePath: aPath on: aCanvas strokeWidth: width
! !

!AthensBalloonStrokePaint methodsFor: 'rendering' stamp: 'NicolaiHess 6/24/2014 22:47'!
athensFillRectangle: aRect on: aCanvas

	^ fillPaint strokeRect: aRect on: aCanvas strokeWidth: width! !

!AthensBalloonStrokePaint methodsFor: 'setting cap styles' stamp: 'NicolaiHess 6/24/2014 22:37'!
capButt
	self flag:#todo! !

!AthensBalloonStrokePaint methodsFor: 'setting cap styles' stamp: 'NicolaiHess 6/24/2014 22:38'!
capRound
	self flag:#todo! !

!AthensBalloonStrokePaint methodsFor: 'setting cap styles' stamp: 'NicolaiHess 6/24/2014 22:38'!
capSquare
	self flag:#todo! !

!AthensBalloonStrokePaint methodsFor: 'setting join styles' stamp: 'NicolaiHess 6/24/2014 22:38'!
joinBevel
	self flag:#todo! !

!AthensBalloonStrokePaint methodsFor: 'setting join styles' stamp: 'NicolaiHess 6/24/2014 22:38'!
joinMiter
	self flag:#todo! !

!AthensBalloonStrokePaint methodsFor: 'setting join styles' stamp: 'NicolaiHess 6/24/2014 22:38'!
joinRound
	self flag:#todo! !

GlyphRenderer subclass: #FT2GlyphRenderer
	instanceVariableNames: 'pixelSize form blt matrix translation penX penY slot surface colorMap bitmapX bitmapY descend origin currentColor hAdvance'
	classVariableNames: ''
	poolDictionaries: 'FT2Constants'
	category: 'Athens-Balloon'!

!FT2GlyphRenderer commentStamp: '<historical>' prior: 0!
i am used to render freetype glyphs using balloon primitives.!

!FT2GlyphRenderer class methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/20/2011 10:29'!
forFont: aFont surface: aSurface
	^ self new initForFont: aFont surface: aSurface! !

!FT2GlyphRenderer methodsFor: 'private' stamp: 'IgorStasenko 10/19/2011 15:31'!
clearBitmap
	
	form bits atAllPut: 0.
! !

!FT2GlyphRenderer methodsFor: 'accessing' stamp: 'IgorStasenko 10/19/2011 15:37'!
form
	^ form! !

!FT2GlyphRenderer methodsFor: 'accessing' stamp: 'IgorStasenko 10/25/2011 18:29'!
getAdvance
	"aPoint is a text origin in user's coordinate system,
	and anvance is accumulated advance came from another renderer instance, expressed in surface's coordinate system"

	| pt |
	pt := (penX/64.0) @ (penY/64.0) - origin.

	^ pt! !

!FT2GlyphRenderer methodsFor: 'initailize-release' stamp: 'IgorStasenko 11/21/2011 16:32'!
initForFont: aFont surface: aSurface

	self assert: (aFont class == FreeTypeFont).
	
	surface := aSurface.
	font := aFont.
	slot := SimpleTextGlyphInfo new.
	
	pixelSize := font pixelSize rounded.

	blt := BitBlt toForm: surface form.

	self loadSurfaceTransform.

	colorMap := Bitmap new: 256.
	
	blt 
		sourceForm: form;
		sourceX: 0;
		sourceY: 0;
		destOrigin: 0@0;
		sourceOrigin: 0@0;
		halftoneForm: nil;
		combinationRule: 24;
		width: form width;
		height: form height;
		colorMap: colorMap;
		clipRect: surface clipRect.
	
	translation := IntegerArray new: 2.
! !

!FT2GlyphRenderer methodsFor: 'private' stamp: 'IgorStasenko 7/12/2011 20:11'!
loadSlotInfo
	slot loadFrom: font face.! !

!FT2GlyphRenderer methodsFor: 'private' stamp: 'IgorStasenko 11/7/2011 14:25'!
loadSurfaceTransform
	| m mt org xaxis yaxis sum xmin xmax ymin ymax formW formH fix face bbox |
	
	face := font face.

	face 
		setPixelWidth: pixelSize height: pixelSize.
	
	bbox := face bbox.
	m := surface pathTransform copy transposed.

	org := m transform: 0@0.
	xaxis := (m transform: bbox right - bbox left * pixelSize / face unitsPerEm @ 0) - org.
	yaxis := (m transform: 0 @ (bbox bottom - bbox top * pixelSize / face unitsPerEm)) - org.
	sum := xaxis + yaxis.
	
	xmin := 0 min: ((xaxis x min: yaxis x) min: sum x).
	ymin := 0 min: ((xaxis y min: yaxis y) min: sum y).
	xmax := 0 max: ((xaxis x max: yaxis x) max: sum x).
	ymax := 0 max: ((xaxis y max: yaxis y) max: sum y).

	formW := (xmax - xmin) ceiling + 1.
	formH := (ymax - ymin) ceiling  + 1.

" freetype 'zero' points to bottom-left point on form" 

	"calculate bitmap translation relative to top-left corner of glyph"
	bitmapX := (yaxis x negated + xmin * 64.0) rounded.
	bitmapY := (xaxis y negated + ymin * 64.0) rounded.

	descend := 0 @  ((face bbox height -  face bbox bottom  * pixelSize / face unitsPerEm) +1).
	descend := (m transform: descend) - org.
	descend := descend + (xmin negated@(ymin negated)).
	descend := (descend * 64 ) rounded.

	form := Form extent: formW @ formH depth: 8.

	blt 
		sourceForm: form;
		width: form width;
		height: form height.

	"prepare transformation matrix for freetype"
	
	matrix := IntegerArray new: 4.

	"values in matrix are 16.16 fixed point floating values" 
	fix := 65536. "(2 raisedTo: 16)."
	
	matrix 
		at: 1 put: (m sx * fix) rounded;
		at: 2 put: (m shx * fix) rounded;
		at: 3 put: (m shy * fix) rounded;
		at: 4 put: (m sy * fix) rounded.

! !

!FT2GlyphRenderer methodsFor: 'private' stamp: 'IgorStasenko 10/25/2011 18:40'!
loadUnicode: unicode

	| ext hintingFlags flags arr face |
	
	face := font face.

"	hintingFlags := FreeTypeSettings current hintingFlags."
	flags :=  LoadNoBitmap bitOr: 2 "hintingFlags".
"	bitOr:( LoadIgnoreTransform  bitOr: 2 ). "
	face primLoadCharacter: unicode flags: flags.

	self loadSlotInfo.
! !

!FT2GlyphRenderer methodsFor: 'private' stamp: 'FernandoOlivero 9/9/2013 19:35'!
pixelValue32Of: aColor 
	^ aColor pixelWordForDepth: 32! !

!FT2GlyphRenderer methodsFor: 'rendering' stamp: 'IgorStasenko 11/7/2011 13:45'!
renderGlyphsIn: text from: start to: stop
	| face |
	face := font face.
"	face
		setPixelWidth: pixelSize height: pixelSize.
"
	start to: stop do: [:i | | bx by |
		
		bx := penX + bitmapX.
		by := penY + bitmapY.

		"add a small shift, to incorporate subpixel position"
		
		translation at: 1 put: descend x + (bx bitAnd: 2r111111).
		translation at: 2 put: descend y - (by bitAnd: 2r111111). 

		face primSetTransform: matrix delta: translation.

		self loadUnicode: (text at: i) asUnicode.
	
		self clearBitmap.
		face renderGlyphIntoForm: form.

		blt
			destX: bx >> 6;
			destY: by >> 6;
			copyBits.

	"increment x by horizontal advance"
		penX := penX + slot advanceX.
		penY := penY - slot advanceY.

	].

! !

!FT2GlyphRenderer methodsFor: 'accessing' stamp: 'FernandoOlivero 9/9/2013 19:36'!
setColor: color
	"Set the color which will be used to render glyphs."
	
	| clr rgb alpha |
	currentColor = color ifTrue: [ ^ self ].
	currentColor := color.
	clr := self pixelValue32Of: color. 
	rgb := clr bitAnd: 16rFFFFFF.
	alpha := clr >> 24.
	
 	0 to: 255 do:[:i | | a |
		a :=  (i+1) * alpha bitAnd: 16rFF00.
		colorMap at: i+1 put: ( (a<<16) + rgb ).
	].

"	colorMap at: 1 put: (Color red  alpha: 0.1) pixelValue32."
! !

!FT2GlyphRenderer methodsFor: 'private' stamp: 'IgorStasenko 11/7/2011 13:45'!
setPosition: aPoint advance: advancePt baseline: baselineOffset
	"aPoint is a text origin in user's coordinate system,
	and andvance is accumulated advance came from another renderer instance, expressed in surface's coordinate system"

	| pt offset |
	
	offset :=  baselineOffset - font getPreciseAscent "(font face ascender  * pixelSize / font face unitsPerEm)".

	pt := aPoint + (0@offset).
	pt := surface pathTransform transform: pt.
	origin := pt.
	pt := pt + advancePt.
	
	"remember the origin, so on #getAdvance , we will answer the current advance for renderer in surface coordinate system"
	
"	2 raisedTo: 6 "
	penX := (pt x * 64) rounded.
	penY := (pt y * 64) rounded.

! !

BalloonEngine subclass: #AthensBalloonEngine
	instanceVariableNames: 'currentFill'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Balloon'!

!AthensBalloonEngine commentStamp: 'IgorStasenko 3/29/2011 01:04' prior: 0!
- balloon engine using only first 4 variables of receiver instance:

 - work buffer
 - span
 - bitBlt (Bitblt object)
 - forms

!

!AthensBalloonEngine methodsFor: 'drawing' stamp: 'IgorStasenko 4/21/2011 16:20'!
copyBits
	(bitBlt notNil and:[bitBlt destForm notNil]) ifTrue:[bitBlt destForm unhibernate].
	self copyLoopFastest.! !

!AthensBalloonEngine methodsFor: 'drawing' stamp: 'IgorStasenko 4/22/2011 13:56'!
drawImage: aForm transform: m 
	| orig dir normal tr corner rect |

	rect := aForm boundingBox.
	orig := 0@0. 
	dir := aForm width @ 0.
	normal := 0 @ aForm height.
	
	tr := edgeTransform.
	edgeTransform := m.

	self prepareForRendering.
	self registerFormFill: aForm
		origin: orig
		direction: dir
		normal: normal.
	self fillRectangle: rect.

	edgeTransform := tr.
! !

!AthensBalloonEngine methodsFor: 'private' stamp: 'IgorStasenko 9/19/2011 16:37'!
fastReset

	workBuffer ifNil: [ ^ self reset ].

	workBuffer 
		at: (1+GWState) put: GEStateUnlocked;
		at: (1+GWObjUsed) put: 4;
		at: (1+GWNeedsFlush) put: 0;
		at: (1+GWCurrentZ) put: 0;
		at: (1+GWGETStart) put: 0";

		at: (1+GWGETUsed) put: 0;
		at: (1+GWAETStart) put: 0;
		at: (1+GWAETUsed) put: 0".
	
	self primSetEdgeTransform: edgeTransform.
	self primSetClipRect: clipRect.
	self primSetColorTransform: colorTransform.
	self primSetDepth: self primGetDepth + 1. 

	postFlushNeeded := false.
	forms := #().

"	
-	self magicNumberPut: GWMagicNumber.
- 	self wbSizePut: size.
+	self wbTopPut: size.
+	self statePut: GEStateUnlocked.
-	self objStartPut: GWHeaderSize.
+	self objUsedPut: 4.	
-	self objectTypeOf: 0 put: GEPrimitiveFill.
-	self objectLengthOf: 0 put: 4.
-	self objectIndexOf: 0 put: 0.

	self getStartPut: 0.
	self getUsedPut: 0.
	self aetStartPut: 0.
	self aetUsedPut: 0.
	self stopReasonPut: 0.

+	self needsFlushPut: 0.
-	self clipMinXPut: 0.
-	self clipMaxXPut: 0.
-	self clipMinYPut: 0.
-	self clipMaxYPut: 0.
+	self currentZPut: 0.
	self resetGraphicsEngineStats.
	self initEdgeTransform.
	self initColorTransform.
"! !

!AthensBalloonEngine methodsFor: 'drawing' stamp: 'IgorStasenko 4/1/2011 09:26'!
fillBezierShape: contours

	contours do:[ :points |
		self primAddBezierShape: points
			segments: (points size // 3)
			fill: currentFill
			lineWidth: 0
			lineFill: 0.
			
		"Note: To avoid premature flushing of the pipeline we need to reset the flush bit within the engine."
		self primFlushNeeded: false.
	].

	self copyBits.! !

!AthensBalloonEngine methodsFor: 'drawing' stamp: 'IgorStasenko 4/21/2011 15:04'!
fillRectangle: rect 

	currentFill = 0 ifTrue: [ ^ self ]. "no fill "
	self primAddRectFrom: rect origin 
			to: rect corner
			fillIndex: currentFill
			borderWidth: 0
			borderColor: 0.

	self copyBits.
	! !

!AthensBalloonEngine methodsFor: 'private' stamp: 'IgorStasenko 4/13/2011 12:15'!
prepareForRendering
	self fastReset! !

!AthensBalloonEngine methodsFor: 'registering fills' stamp: 'IgorStasenko 4/1/2011 09:16'!
registerColorFill: aColor 

	currentFill := aColor scaledPixelValue32.
! !

!AthensBalloonEngine methodsFor: 'registering fills' stamp: 'NicolaiHess 6/24/2014 23:18'!
registerFormFill: form origin: orig direction: dir normal: norm
	self
		registerFormFill: form
		origin: orig
		direction: dir
		normal: norm
		tileFlag: true! !

!AthensBalloonEngine methodsFor: 'registering fills' stamp: 'NicolaiHess 6/24/2014 23:18'!
registerFormFill: form origin: orig direction: dir normal: norm tileFlag: tileFlag

	forms := forms copyWith: form.
	form unhibernate.
	currentFill := self primAddBitmapFill: form
			colormap: (form colormapIfNeededForDepth: 32)
			tile: tileFlag
			from: orig
			along: dir
			normal: norm
			xIndex: forms size.
			
! !

!AthensBalloonEngine methodsFor: 'paints' stamp: 'IgorStasenko 4/1/2011 16:01'!
registerLinearGradient: colorRamp origin: origin corner: corner 
	| pixelRamp direction normal |
	
	pixelRamp := GradientFillStyle pixelRampCache at: colorRamp.
	direction := corner - origin.
	normal := direction y negated @ direction x.
	
	currentFill := 
		self primAddGradientFill: pixelRamp
			from: origin
			along: direction
			normal:  normal
			radial:  false.! !

!AthensBalloonEngine methodsFor: 'paints' stamp: 'IgorStasenko 4/14/2011 17:15'!
registerRadialGradient: colorRamp center: aCenter direction: dir normal: n 
	| pixelRamp |
	
	pixelRamp := GradientFillStyle pixelRampCache at: colorRamp.
	
	currentFill := 
		self primAddGradientFill: pixelRamp
			from: aCenter
			along: dir
			normal:  n
			radial:  true.! !

!AthensBalloonEngine methodsFor: 'paints' stamp: 'IgorStasenko 4/14/2011 16:26'!
registerRadialGradient: colorRamp center: aCenter radius: r 
	| pixelRamp |
	
	pixelRamp := GradientFillStyle pixelRampCache at: colorRamp.
	
	currentFill := 
		self primAddGradientFill: pixelRamp
			from: aCenter
			along: (aCenter x + r @ aCenter y)
			normal:  (aCenter x  @ aCenter y - r)
			radial:  true.! !

!AthensBalloonEngine methodsFor: 'initialize' stamp: 'IgorStasenko 4/22/2011 14:00'!
reset
	workBuffer ifNil:[workBuffer := self class allocateOrRecycleBuffer: 10000].
	self primInitializeBuffer: workBuffer.
	self primSetAALevel: 4.
	self primSetOffset: 0@0.
	self primSetClipRect: clipRect.
	self primSetEdgeTransform: edgeTransform.
	self primSetColorTransform: colorTransform.
	forms := #().! !

!AthensBalloonEngine methodsFor: 'accessing' stamp: 'IgorStasenko 4/1/2011 09:03'!
setFill: aFill
	currentFill := aFill! !

!AthensBalloonEngine methodsFor: 'private' stamp: 'IgorStasenko 9/14/2011 15:12'!
strokeBezierShape: contours width: aWidth

	contours do:[ :points |
		self primAddBezierShape: points
			segments: (points size // 3)
			fill: 0
			lineWidth: aWidth
			lineFill: currentFill.
			
		"Note: To avoid premature flushing of the pipeline we need to reset the flush bit within the engine."
		self primFlushNeeded: false.
	].

	self copyBits. 
! !

!AthensBalloonEngine methodsFor: 'drawing' stamp: 'IgorStasenko 5/2/2011 04:36'!
strokeRectangle: rect borderWidth: width

	currentFill = 0 ifTrue: [ ^ self ]. "no fill "
	self primAddRectFrom: rect origin 
			to: rect corner
			fillIndex: 0
			borderWidth: width
			borderColor: currentFill.

	self copyBits.
	! !

!AthensBalloonEngine methodsFor: 'drawing' stamp: 'IgorStasenko 9/19/2011 15:38'!
strokeRectangle: rect width: aWidth

	currentFill = 0 ifTrue: [ ^ self ]. "no fill "
	self primAddRectFrom: rect origin 
			to: rect corner
			fillIndex: 0
			borderWidth: aWidth asInteger
			borderColor: currentFill.

	self copyBits.
	! !

AthensPath subclass: #AthensBalloonPath
	instanceVariableNames: 'bounds contours last'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Balloon'!

!AthensBalloonPath commentStamp: '<historical>' prior: 0!
i represent a path shape in balloon backend!

!AthensBalloonPath methodsFor: 'accessing' stamp: 'IgorStasenko 3/27/2011 00:23'!
contours
	
	^ contours! !

!AthensBalloonPath methodsFor: 'converting' stamp: 'IgorStasenko 3/29/2011 14:53'!
contoursForFilling
	"answer a collection of contours, ready for filling by balloon engine"
	^ contours collect: [:contour |
		contour last = contour first 
		ifTrue: [contour asArray] 
		ifFalse: [ (contour , { contour last. contour last. contour first }) ]
	]! !

!AthensBalloonPath methodsFor: 'converting' stamp: 'IgorStasenko 9/14/2011 15:08'!
contoursForStroke
	"answer a collection of contours, ready for stroking by balloon engine"
	^ contours collect: [:contour | contour asArray ]! !

!AthensBalloonPath methodsFor: 'converting' stamp: 'IgorStasenko 3/29/2011 14:55'!
convertFromBuilder: aBuilder
	| segment cContours |
	bounds := aBuilder pathBounds.
	segment := aBuilder pathStart.
	

	[ segment notNil ] whileTrue: [
		segment visitWith: self.
		segment := segment next ].
	
	" finally make all OrderedColllections ==> Array "
	cContours := Array new: contours size.
	1 to: cContours size do: [:i |
		cContours at: i put: (contours at: i) asArray.
		].
	contours := cContours.! !

!AthensBalloonPath methodsFor: 'private' stamp: 'IgorStasenko 3/27/2011 17:40'!
duplicateLastPoint

	contours last size < 2 ifFalse: [
		"duplicate last contour point"
		contours last add: (contours last last).
		]. 
! !

!AthensBalloonPath methodsFor: 'accessing' stamp: 'IgorStasenko 3/27/2011 13:54'!
numContours
	^ contours size! !

!AthensBalloonPath methodsFor: 'drawing' stamp: 'IgorStasenko 8/30/2013 16:32'!
paintFillsUsing: aPaint on: anAthensCanvas 
	
	"This method is a part of rendering dispatch  Canvas->receiver->paint"
	
	^ aPaint athensFillPath: self on: anAthensCanvas! !

!AthensBalloonPath methodsFor: 'drawing' stamp: 'IgorStasenko 9/13/2011 16:21'!
paintStrokeUsing: paint on: anAthensCanvas 
	
	^ paint strokePath: self on: anAthensCanvas
! !

!AthensBalloonPath methodsFor: 'converting' stamp: 'IgorStasenko 3/27/2011 17:31'!
visitCloseSegment: closeSegment
	
	"see if its alredy closed"
	contours last last = contours last first ifFalse: [
		
		contours last 
			add: (contours last last);
			add: closeSegment endPoint;
			add: closeSegment endPoint
	]! !

!AthensBalloonPath methodsFor: 'converting' stamp: 'IgorStasenko 3/27/2011 17:40'!
visitLineSegment: line
	self duplicateLastPoint.
	contours last add: line endPoint;
		add: line endPoint.! !

!AthensBalloonPath methodsFor: 'converting' stamp: 'IgorStasenko 3/27/2011 00:05'!
visitMoveSegment: seg
	
	contours ifNil: [
		contours := OrderedCollection new ].
	
	contours add: OrderedCollection new.
	contours last add: seg endPoint.! !

!AthensBalloonPath methodsFor: 'converting' stamp: 'IgorStasenko 3/27/2011 17:40'!
visitQuadSegment: quadSegment

	self duplicateLastPoint.

	contours last add: quadSegment via;
		add: quadSegment to! !
